import { EOrientation } from '../../types'
import { Page } from './Page'
import { AdaptableTree } from '../Tree/AdaptableTree'
import { AdaptableTable } from '../Table/AdaptableTable'

import * as utils from '../../utils'

export class Pages {
  private pages: Page[] = []
  private orientation: EOrientation
  private templatePage: string

  constructor(orientation: EOrientation, templatePage: string) {
    this.pages.push(new Page(orientation, templatePage))
    this.templatePage = templatePage
    this.orientation = orientation
  }

  private createPage() {
    const page = new Page(this.orientation, this.templatePage)
    this.pages.push(page)
    return page
  }

  get lastPage() {
    return this.pages[this.pages.length - 1]
  }
  get pagesContent() {
    return this.pages.reduce((accum, page) => accum + page.pageContent, '')
  }

  async insertElem(elem: HTMLDivElement) {
    const type = utils.identificationRowElem(elem)


    switch (type) {
      case 'simple':
        this.insertSimpleElem(elem)
        break;
      case 'image':
        await this.insertImage(elem)
        break
      case 'table':
        this.insertTable(elem)
        break
    }
  }

  private insertSimpleElem(elem: HTMLDivElement) {

    if (elem.offsetHeight <= this.lastPage.height.balanceHeight) {
      this.lastPage.insertElem(elem)
      return
    }

    const rowElem = new AdaptableTree(elem)

    while (!rowElem.isEmpty) {

      if (!this.lastPage.height.balanceHeight) {
        this.createPage()
        continue
      }

      const tree = rowElem.createTree(this.lastPage.height.balanceHeight)

      if (!tree.root.offsetHeight || !tree.isHaveContent) {
        this.createPage()
        continue
      }

      this.lastPage.insertElem(tree.root)
    }
  }

  private insertTable(elem: HTMLDivElement) {


    const tables = new AdaptableTable(elem, this.orientation)
      .getTables(
        this.lastPage.height.balanceHeight,
        this.lastPage.height.fullHeight
      )
    tables.forEach((table, index) => {
      this.lastPage.insertElem(table)
      index != tables.length - 1 && this.createPage()
    })
  }

  private async insertImage(elem: HTMLDivElement) {
    // const elem = _elem.cloneNode(true) as HTMLDivElement
    const img = elem.querySelector('img')!
    await new Promise<void>((resolve, reject) => {
      img.onload = () => resolve()
      setTimeout(() => { reject('Не могу высчитать высоту картинки, слишком долго грузится') }, 60000)
    })
    this.insertSimpleElem(elem)
  }

}