import { EOrientation } from "../../types";
import { AdaptableTree } from "../Tree/AdaptableTree";
import { TableShrink } from "./TableShrink";
import { TableTransfrom } from "./TableTransfrom";
import { TableUtils } from "./TableUtils";


type TypeCreateTable = 'new' | 'continue'

export class AdaptableTable {
  private _table: HTMLDivElement
  private _fullHeightTable: number
  private _tHeadHeight: number

  private _tableWrapperOriginal: HTMLDivElement
  private _tHeadOriginal: HTMLDivElement
  private _tBodyRowsOriginal: HTMLDivElement[]

  constructor(elem: HTMLDivElement, orientation: EOrientation) {


    this._table = TableTransfrom.transform(elem)
    TableShrink.shrink(this._table, orientation)

    this._fullHeightTable = this._table.offsetHeight
    this._tHeadHeight = (this._table.querySelector('.table-header') as HTMLDivElement).offsetHeight

    this._tableWrapperOriginal = this._table.querySelector('.table')! as HTMLDivElement
    this._tHeadOriginal = this._table.querySelector('.table-header')! as HTMLDivElement
    this._tBodyRowsOriginal = [...this._table.querySelectorAll('.table-body .table-row')] as HTMLDivElement[]
  }

  // Получаем все таблицы, где каждая влезает на страницу
  getTables(currentBalanceHeight: number, fullHeightPage: number): HTMLDivElement[] {

    if (this._fullHeightTable < currentBalanceHeight)
      return [this._table]

    const resTables: HTMLDivElement[] = []

    // Тип таблицы, которую нужно создать
    let typeCreate: 'new' | 'continue' = 'new'
    let isFirstPage: boolean = true

    while (this._tBodyRowsOriginal.length) {

      const maxHeight = isFirstPage ? currentBalanceHeight : fullHeightPage

      const [table, nextTypeCreate] = this.getTable(maxHeight, typeCreate)

      isFirstPage = false
      typeCreate = nextTypeCreate

      resTables.push(table)
    }

    return resTables
  }


  private getTable(maxHeight: number, typeCreate: TypeCreateTable): [HTMLDivElement, TypeCreateTable] {
    const table = this._tableWrapperOriginal.cloneNode() as HTMLDivElement

    typeCreate == 'new' && table.appendChild(this._tHeadOriginal.cloneNode(true))

    TableUtils.insertInHiddenDom(table)

    // тип следующей таблицы
    const typeNextCreate = this.fillTable(maxHeight - this._tHeadHeight, table)

    return [table, typeNextCreate]
  }


  // Таблица уже должна быть в DOM для вычисления корректной высоты
  // Добавляет элементы в table
  private fillTable(maxHeight: number, table: HTMLDivElement): TypeCreateTable {

    // тип следующей таблицы
    let nextTypeCreate: TypeCreateTable = 'new'

    const tableBody = TableUtils.createTableElement('table-body')
    table.appendChild(tableBody)

    // Значит что больше вставлять строки не нужно
    let stoplose = false

    this._tBodyRowsOriginal = this._tBodyRowsOriginal.reduce((accum, tableRow) => {

      if (stoplose) {
        return [...accum, tableRow]
      }

      if (tableRow.offsetHeight + table.offsetHeight < maxHeight) {
        tableBody.appendChild(tableRow.cloneNode(true))
        return accum
      }


      if (tableRow.offsetHeight + table.offsetHeight > maxHeight && maxHeight - table.offsetHeight > 120) {

        // Если мы делим ячейку, то следующая таблица это продолжение предыдущей
        nextTypeCreate = 'continue'
        stoplose = true

        const [startRow, endRow] = this.splitRow(maxHeight - table.offsetHeight, tableRow)
        startRow && tableBody.appendChild(startRow)
        endRow && accum.push(endRow)

        return accum
      }

      stoplose = true

      return [...accum, tableRow]
    }, [] as HTMLDivElement[])

    return nextTypeCreate
  }

  splitRow(maxHeight: number, row: HTMLDivElement) {

    // Первая часть таблицы
    const startRow = row.cloneNode() as HTMLDivElement
    // Остатки таблицы, которые не влезли на страницу
    const endRow = row.cloneNode() as HTMLDivElement

    // Для получения в будущем верной высоты элементов
    // Вставляем остатки от таблицы в дом
    TableUtils.insertInHiddenDom(endRow)

    const cells = [...row.querySelectorAll('.table-cell')] as HTMLDivElement[]
    const endCellsTreeCreator: AdaptableTree[] = []

    cells.forEach((cell) => {
      // Нам нужно с каждой ячейки взять только часть данных
      // Остальную часть оставить, поэтому для каждой ячейки дергаем AdaptableTree 
      const treeCreator = new AdaptableTree(cell)
      const tableCellEndPage = treeCreator.createTree(maxHeight).root

      tableCellEndPage.classList.add('table-cell-end-page')

      startRow.appendChild(tableCellEndPage)

      // Собираем все остатки от ячеек
      // Чтобы если хотя бы из в одной ячейке будуд данные
      // то все остальные ячейки нужно будет так же создать
      endCellsTreeCreator.push(treeCreator)
    })

    if (endCellsTreeCreator.find((treeCreator) => treeCreator.isHaveContent)) {
      endCellsTreeCreator.forEach(treeCreator => {
        treeCreator.root.classList.add('table-cell-part')
        treeCreator.root.classList.add('table-cell-start-page')
        endRow.appendChild(treeCreator.root)
      })
    }

    return [
      startRow,
      endRow.childNodes.length ? endRow : null
    ]
  }

}