import { UniqueTextTree } from "./UniqueTextTree";
import { uniqueId } from '../../utils/uniqueId'
import { Tree } from "./Tree";


export class AdaptableTree extends UniqueTextTree {

  constructor(root: HTMLDivElement) {
    super(root)
  }

  public createTree(maxHeight: number) {
    const cloneDomTree = this.createCloneTree(maxHeight)

    const tree = new Tree(cloneDomTree)

    this.removeUsedTextNodeFromOriginalTree(cloneDomTree)

    this.removeEmptyBranch()

    return tree
  }

  // Знание 1. 
  // При начальных условиях все наши листья это textNode из-за UniqueTreeText
  // Знание 2. 
  // На каждой ветви колбек в первый раз сработает на textNode
  private createCloneTree(maxHeight: number) {
    const cloneTree = this.root.cloneNode(true) as HTMLDivElement

    const iframe = document.querySelector('.iframe-hidden-dom') as HTMLIFrameElement
    iframe.contentDocument?.body.querySelector('.hidden-dom .template-constructor')?.appendChild(cloneTree)

    const removeNode = (node: HTMLElement) => {

      // Если это лист и он не текстовый то удаляем его
      // Такое бывает в том случае, если на предыдущем сработанном колбеке рекурсии
      // текстова нода не влезла в размеры и была удалена, а это ее родитель
      // 'IMG' - появилось в связи и использованием алгоритма в таблице
      if (node.childNodes.length == 0 && node.nodeType !== 3 && node.nodeName !== 'IMG') {
        node.remove()
        return
      }

      if (cloneTree.offsetHeight > maxHeight) {

        const cloneNode = document.createTextNode('')
        const parent = node.parentElement
        const uniqueDfs = parent?.classList.value.split(' ').find(c => c.includes('dfs'))!
        const textContent = node.textContent

        node.remove()

        // Проверяем после удаления нашу высоту, потому что текстовая нода не имеет высоты
        // Но влияет на высоту дерева
        if (cloneTree.offsetHeight < maxHeight && textContent) {

          parent?.appendChild(cloneNode)

          textContent?.split('').some((c) => {

            const prevContent = cloneNode.textContent
            cloneNode.textContent = cloneNode.textContent!.concat(c)

            if (cloneTree.offsetHeight > maxHeight) {
              cloneNode.textContent = prevContent
              return true
            }
          })

          const origParent = this.root.querySelector(`.${uniqueDfs}`)
          const origTextNode = origParent?.childNodes[0]

          origParent?.classList.remove(uniqueDfs)
          origParent?.classList.add(uniqueId.generateId('dfs'))

          origTextNode!.textContent = textContent.replace(cloneNode.textContent!, '')
        }
      }
    }

    this.dfs({
      node: cloneTree,
      settings: {
        isRevertDfs: true,
        callback: removeNode
      }
    })

    return cloneTree
  }

  // Если текстовая нода используется в клоне дерева, то по уникальной обертке
  // идентивицирующей нахождения текста в оригинальном дереве
  // созданной через UniqueTextTree находим эту обертку в оригальном дереве
  // и удаляем вместе с текстом  
  private removeUsedTextNodeFromOriginalTree(cloneTree: HTMLDivElement) {

    const removeUsedNode = (node: HTMLElement) => {
      if (node.nodeType == 3) {
        const parent = node.parentElement!
        const currentDfsKey = parent.classList.value.split(' ').filter((v) => v.includes('dfs'))
        this.root.querySelector(`.${currentDfsKey}`)?.remove()
      }

      if (node.nodeName == 'IMG') {
        const currentDfsKey = node.classList.value.split(' ').filter((v) => v.includes('dfs'))
        this.root.querySelector(`.${currentDfsKey}`)?.remove()
      }
    }

    this.dfs({
      node: cloneTree,
      settings: { callback: removeUsedNode }
    })
  }

  // Если лист не текстовая нода, то удаляем всю ветвь
  private removeEmptyBranch() {

    const removeEmptyChild = (node: HTMLElement) => {
      if (node.nodeType !== 3 && node.nodeName !== 'IMG' && !node.childNodes.length) {
        node.remove()
      }
    }

    this.dfs({
      node: this.root,
      settings: { callback: removeEmptyChild }
    })
  }
}