
type DfsData = {
  node: HTMLElement,
  settings?: {
    callback?: (node: HTMLElement) => void
    isRevertDfs?: boolean
  }
}

export class Tree {
  private _root: HTMLDivElement

  constructor(root: HTMLDivElement) {
    this._root = root
  }

  get isEmpty(): boolean {
    return !this._root.childNodes.length
  }

  get isHaveContent(): boolean {
    let isHaveTextContent = false

    this.dfs({
      node: this._root,
      settings: {
        callback: (node) => {
          if (node.textContent?.length) {
            isHaveTextContent = true
          }
        }
      }
    })

    const isHaveSign = !!this._root.querySelector('img')

    return isHaveTextContent || isHaveSign
  }

  get countNodes(): number {

    let count = 0

    this.dfs({
      node: this._root,
      settings: { callback: () => count++ }
    })

    return count
  }

  set root(value: HTMLDivElement) {
    this._root = value
  }
  get root() {
    return this._root
  }

  public dfs({ node, settings }: DfsData) {
    const childrens = [...node.childNodes] as HTMLElement[]
    settings?.isRevertDfs && childrens.reverse()

    childrens.forEach((node) => this.dfs({ node, settings }))

    settings?.callback?.(node)
  }
}