import { Tree } from "./Tree";
import { uniqueId } from '../../utils/uniqueId'


// Трансформируем наше дерево по правилу
// "текстовая нода не может быть с кем-то в близких отношениях"
// то есть у нее не должно быть братьев
// поэтому если мы находим такую ноду
// то создаем под нее нового родителя и прикрепляем текст к этому родителю
// Сделано это для того, чтобы каждый текст был уникально идентифицирован по его родителю

export class UniqueTextTree extends Tree {

  constructor(root: HTMLDivElement) {
    super(root)
    this.root = this.transformTree(root)
  }

  private transformTree(root: HTMLDivElement) {

    const cloneRoot = root.cloneNode(true) as HTMLDivElement

    const iframe = document.querySelector('.iframe-hidden-dom') as HTMLIFrameElement
    iframe.contentDocument?.body.querySelector('.hidden-dom .template-constructor')?.appendChild(cloneRoot)

    const dfsCallback = (node: HTMLElement) => {

      const classId = uniqueId.generateId('dfs')

      const parent = node.parentElement

      if (node.nodeType !== 3 || !parent) {
        node.classList.add(classId)
        return
      }

      if (parent!.childNodes.length > 1 || (parent.tagName == 'DIV' && node.nodeType == 3)) {
        const wrapNode = document.createElement('span')

        wrapNode.appendChild(node.cloneNode(true))
        wrapNode.classList.add(classId)

        parent!.insertBefore(wrapNode, node)

        node.remove()
      }
    }

    this.dfs({
      node: cloneRoot,
      settings: { callback: dfsCallback }
    })

    return cloneRoot
  }
}